/*
  CPSPC'04
  Task "triangles"
  O(N^2) solution by Marek Sulovsky
*/
#include <stdio.h>
#include <stdlib.h>

#define MAXN      10240
#define INFTY  0x7fffffff

struct triangle {
  int x, y, d;
} t[MAXN];

struct list {
  int id;
  int next, prev;
} l[MAXN];

struct crosslist {
  int x;
  int next, prev;
} c[4*MAXN];

int N, area = 0;
int lfirst= -1, lsize = 0;
int clf = -1, clsize = 0;

int
txcmp (const void *ca, const void *cb)
{
  struct triangle *a = (struct triangle *)ca,
    *b = (struct triangle *)cb;

  return (a->x - b->x < 0) ? -1 : (a->x == b->x) ? 0 : 1;
}

int
list_add (int id)
{
  int actl, prevl = -1, actp;

  if (lfirst == -1 || t[l[lfirst].id].y >= t[id].y) {
    actl = lfirst;
    prevl = -1;
    l[lsize].next = lfirst;
    l[lsize].prev = -1;
    if (lfirst != -1) l[lfirst].prev = lsize;
    l[lsize].id = id;
    actp = lsize++;
    lfirst = actp;
  } else {
    for (actl = lfirst; actl != -1 && t[l[actl].id].y < t[id].y; actl = l[actl].next) 
      prevl = actl;
    l[lsize].next = actl;
    l[lsize].prev = prevl;
    if (prevl != -1) l[prevl].next = lsize;
    if (actl != -1) l[actl].prev = lsize;
    l[lsize].id = id;
    actp = lsize++;
  }
  /* check containment */
  if (prevl != -1 &&
      t[id].y + t[id].d <
      t[l[prevl].id].y + t[l[prevl].id].d - (t[id].x - t[l[prevl].id].x)) {
    /* I am contained - so I should be removed */
    if (lfirst == actp) {
      lfirst = l[actp].next;
      if (lfirst != -1) l[lfirst].prev = -1;
    } else {
      l[prevl].next = actl;
      if (actl != -1) l[actl].prev = prevl;
    }
    return -1;
  }
  /* if I contain other, delete them */
  while (actl != -1 &&
	 t[id].y + t[id].d >=
	 t[l[actl].id].y + t[l[actl].id].d - (t[id].x - t[l[actl].id].x)) {
    actl = l[actl].next;
    l[actp].next = actl;
    if (actl != -1) l[actl].prev = actp;
  }
  return actp;
}

void
clist_add (int x)
{
  int prevc = -1, actc;

  if (clf == -1 || c[clf].x > x) {
    c[clsize].x = x;
    c[clsize].next = clf;
    if (clf != -1) c[clf].prev = clsize;
    c[clsize].prev = -1;
    clf = clsize++;
    return;
  }
  for (actc = clf; actc != -1 && c[actc].x < x; actc = c[actc].next) 
    prevc = actc;
  if (actc != -1 && c[actc].x > x) {
    c[clsize].next = actc;
    c[clsize].prev = prevc;
    if (prevc != -1) c[prevc].next = clsize;
    c[actc].prev = clsize;
    c[clsize].x = x;
    clsize++;
  } else if (actc == -1) {
    c[clsize].next = -1;
    c[clsize].prev = prevc;
    if (prevc != -1) c[prevc].next = clsize;
    c[clsize].x = x;
    clsize++;
  }
}

int
main (void)
{
  FILE *fin = fopen ("triangles.in", "r"),
    *fout = fopen ("triangles.out", "w");
  int i, k;
  int prevx, upper, lower;
  int from, to, nf, nt;

  /* Processing input & initializing */
  fscanf (fin, "%d", &N);
  for (i = 0; i < N; i++)
    fscanf (fin, "%d%d%d", &t[i].x, &t[i].y, &t[i].d);
  qsort (t, N, sizeof(struct triangle), txcmp);

  prevx = t->x;
  t[N].x = INFTY;
  for (i = N-1; i >= 0; i--)
    clist_add (t[i].x);
  /* for each add event */
  for (i = 0; i <= N; i++)
    {
      /* for each cross or end event preceding our add */
      for (; lfirst != -1 && clf != -1 && c[clf].x <= t[i].x; clf = c[clf].next)
	{
	  c[clf].prev = -1;
	  /* compute area */
	  for (k = lfirst; k != -1; )
	    {
	      from = t[l[k].id].y;
	      to = t[l[k].id].y + t[l[k].id].d + t[l[k].id].x - c[clf].x;
	      nt = to;
	      do {
		to = nt;
		k = l[k].next;
		if (k == -1)
		  break;
		nf = t[l[k].id].y;
		nt = t[l[k].id].y + t[l[k].id].d + t[l[k].id].x - c[clf].x;
	      } while (nf <= to);
	      /* area is in twice as small units */
	      area += 2*(to - from) * (c[clf].x - prevx) +
		(c[clf].x - prevx) * (c[clf].x - prevx);
	    }
	  for (k = lfirst; k != -1; k = l[k].next)
	    {
	      /* check whether it is necessary to delete this triangle */
	      if (t[l[k].id].x + t[l[k].id].d == c[clf].x) {
		if (l[k].prev == -1) {
		  lfirst = l[k].next;
		  if (l[k].next != -1) l[l[k].next].prev = -1;
		} else {
		  l[l[k].prev].next = l[k].next;
		  if (l[k].next != -1) l[l[k].next].prev = l[k].prev;
		}
	      }
	    }
	  prevx = c[clf].x;
	}
      if (i == N)
	break;
      prevx = t[i].x;
      /* find position */
      k = list_add (i);
      /* add endpoint */
      clist_add (t[i].x + t[i].d);
      /* add crossings */
      upper = i;
      if (k != -1 && l[k].prev != -1) {
	lower = l[l[k].prev].id;
	if (t[lower].y + t[lower].d - (t[upper].x - t[lower].x) > t[upper].y)
	  clist_add (t[lower].x + t[lower].d - (t[upper].y - t[lower].y));
      }
      lower = upper;
      if (k != -1 && l[k].next != -1) {
	upper = l[l[k].next].id;
	if (t[lower].y + t[lower].d > t[upper].y)
	  clist_add (t[lower].x + t[lower].d - (t[upper].y - t[lower].y));
      }
    }

  fprintf(fout, "%d.%d\n", area / 2, (area % 2) * 5);
  fclose (fin);
  fclose (fout);
  return EXIT_SUCCESS;
}
